@@ -31,12 +31,11 @@ struct _GgitDiffLine {
 	gint old_lineno;
 	gint new_lineno;
 	gint num_lines;
-	gsize content_len;
 	gint64 content_offset;
-	const gchar *content;
+	GBytes *content;
 
 	gchar *text;
-	const gchar *encoding;
+	gchar *encoding;
 };
 
 G_DEFINE_BOXED_TYPE (GgitDiffLine, ggit_diff_line,
@@ -56,10 +55,9 @@ _ggit_diff_line_wrap (const git_diff_line *line,
 	gline->origin = (GgitDiffLineType)line->origin;
 	gline->old_lineno = line->old_lineno;
 	gline->new_lineno = line->new_lineno;
-	gline->content_len = line->content_len;
 	gline->content_offset = line->content_offset;
-	gline->content = line->content;
-	gline->encoding = encoding;
+	gline->content = g_bytes_new (line->content, line->content_len);
+	gline->encoding = g_strdup (encoding);
 	gline->text = NULL;
 
 	return gline;
@@ -72,7 +70,7 @@ _ggit_diff_line_wrap (const git_diff_line *line,
  * Atomically increments the reference count of @line by one.
  * This function is MT-safe and may be called from any thread.
  *
- * Returns: a #GgitDiffLine.
+ * Returns: (transfer none) (nullable): a #GgitDiffLine or %NULL.
  **/
 GgitDiffLine *
 ggit_diff_line_ref (GgitDiffLine *line)
@@ -98,6 +96,7 @@ ggit_diff_line_unref (GgitDiffLine *line)
 
 	if (g_atomic_int_dec_and_test (&line->ref_count))
 	{
+		g_bytes_unref (line->content);
 		g_free (line->text);
 		g_slice_free (GgitDiffLine, line);
 	}
@@ -184,10 +183,10 @@ ggit_diff_line_get_content (GgitDiffLine *line,
 
 	if (length)
 	{
-		*length = line->content_len;
+		*length = g_bytes_get_size (line->content);
 	}
 
-	return (const guint8 *)line->content;
+	return g_bytes_get_data (line->content, NULL);
 }
 
 /**
@@ -196,7 +195,7 @@ ggit_diff_line_get_content (GgitDiffLine *line,
  *
  * Get the content of the diff line as UTF-8 encoded text.
  *
- * Returns: the content in utf-8 encoding.
+ * Returns: (transfer none) (nullable): the content in utf-8 encoding or %NULL.
  *
  **/
 const gchar *
@@ -206,8 +205,12 @@ ggit_diff_line_get_text (GgitDiffLine *line)
 
 	if (line->text == NULL)
 	{
-		line->text = ggit_convert_utf8 (line->content,
-		                                line->content_len,
+		const gchar *content;
+		gsize content_len;
+
+		content = g_bytes_get_data (line->content, &content_len);
+		line->text = ggit_convert_utf8 (content,
+		                                content_len,
 		                                line->encoding);
 	}
 
