@@ -208,12 +208,17 @@ ggit_diff_file_callback_wrapper (const git_diff_delta *delta,
 
 			path = ggit_diff_file_get_path (file);
 
-			data->encoding =
-				ggit_repository_get_attribute (priv->repository,
-			                                       path,
-			                                       "encoding",
-			                                       GGIT_ATTRIBUTE_CHECK_FILE_THEN_INDEX,
-			                                       NULL);
+			if (priv->repository != NULL)
+			{
+				data->encoding =
+					ggit_repository_get_attribute (priv->repository,
+				                                       path,
+				                                       "encoding",
+				                                       GGIT_ATTRIBUTE_CHECK_FILE_THEN_INDEX,
+				                                       NULL);
+			} else {
+				data->encoding = "UTF-8";
+			}
 		}
 	}
 
@@ -665,7 +670,7 @@ ggit_diff_foreach (GgitDiff              *diff,
 	git_diff_line_cb real_line_cb = NULL;
 
 	g_return_if_fail (GGIT_IS_DIFF (diff));
-	g_return_if_fail (file_cb != NULL && binary_cb != NULL && hunk_cb != NULL && line_cb != NULL);
+	g_return_if_fail (file_cb != NULL || binary_cb != NULL || hunk_cb != NULL || line_cb != NULL);
 	g_return_if_fail (error == NULL || *error == NULL);
 
 	wrapper_data_init (&wrapper_data);
@@ -963,14 +968,14 @@ ggit_diff_blobs (GgitBlob              *old_blob,
  * there was no error, %NULL otherwise.
  */
 GgitDiff *
-ggit_diff_new_buffers (const guint8          *buffer1,
-                   gssize                 buffer1_len,
-                   const gchar           *buffer1_as_path,
-                   const guint8          *buffer2,
-                   gssize                 buffer2_len,
-                   const gchar           *buffer2_as_path,
-                   GgitDiffOptions       *diff_options,
-                   GError               **error)
+ggit_diff_new_buffers (const guint8    *buffer1,
+                       gssize           buffer1_len,
+                       const gchar     *buffer1_as_path,
+                       const guint8    *buffer2,
+                       gssize           buffer2_len,
+                       const gchar     *buffer2_as_path,
+                       GgitDiffOptions *diff_options,
+                       GError          **error)
 {
 
 
@@ -1047,18 +1052,18 @@ ggit_diff_new_buffers (const guint8          *buffer1,
  * Same as ggit_diff_blobs() but using a buffer.
  */
 void
-ggit_diff_blob_to_buffer (GgitBlob              *old_blob,
-                          const gchar           *old_as_path,
-                          const guint8          *buffer,
-                          gssize                 buffer_len,
-                          const gchar           *buffer_as_path,
-                          GgitDiffOptions       *diff_options,
-                          GgitDiffFileCallback   file_cb,
-                          GgitDiffBinaryCallback binary_cb,
-                          GgitDiffHunkCallback   hunk_cb,
-                          GgitDiffLineCallback   line_cb,
-                          gpointer               user_data,
-                          GError               **error)
+ggit_diff_blob_to_buffer (GgitBlob                *old_blob,
+                          const gchar             *old_as_path,
+                          const guint8            *buffer,
+                          gssize                   buffer_len,
+                          const gchar             *buffer_as_path,
+                          GgitDiffOptions         *diff_options,
+                          GgitDiffFileCallback     file_cb,
+                          GgitDiffBinaryCallback   binary_cb,
+                          GgitDiffHunkCallback     hunk_cb,
+                          GgitDiffLineCallback     line_cb,
+                          gpointer                 user_data,
+                          GError                 **error)
 {
 	gint ret;
 	const git_diff_options *gdiff_options;
